var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

function bindArenaActionCreator(actionCreator, dispatch, sceneReducerKey) {
  return function () {
    var action = actionCreator.apply(undefined, arguments);
    if (action && action._sceneReducerKey) {
      console.warn('"Action with redux-arena should not contain an user specified "_sceneReducerKey" property.\n' + ("Occurred in type: " + action.type + ", _sceneReducerKey: " + _sceneReducerKey + "."));
    }
    (typeof action === "undefined" ? "undefined" : _typeof(action)) === "object" ? dispatch(Object.assign({}, { _sceneReducerKey: sceneReducerKey }, action)) : dispatch(action);
  };
}

/**
 * Copy from redux/src/bindActionCreators.js, for binding scene pointer.
 * 
 * Turns an object whose values are action creators, into an object with the
 * same keys, but with every function wrapped into a `dispatch` call so they
 * may be invoked directly. This is just a convenience method, as you can call
 * `store.dispatch(MyActionCreators.doSomething())` yourself just fine.
 *
 * For convenience, you can also pass a single function as the first argument,
 * and get a function in return.
 *
 * @param {Function|Object} actionCreators An object whose values are action
 * creator functions. One handy way to obtain it is to use ES6 `import * as`
 * syntax. You may also pass a single function.
 *
 * @param {Function} dispatch The `dispatch` function available on your Redux
 * store.
 *
 * @returns {Function|Object} The object mimicking the original object, but with
 * every action creator wrapped into the `dispatch` call. If you passed a
 * function as `actionCreators`, the return value will also be a single
 * function.
 */
export default function bindArenaActionCreators(actionCreators, dispatch, sceneReducerKey) {
  if (typeof actionCreators === "function") {
    return bindArenaActionCreator(actionCreators, dispatch);
  }

  if ((typeof actionCreators === "undefined" ? "undefined" : _typeof(actionCreators)) !== "object" || actionCreators === null) {
    throw new Error("bindArenaActionCreators expected an object or a function, instead received " + (actionCreators === null ? "null" : typeof actionCreators === "undefined" ? "undefined" : _typeof(actionCreators)) + ". " + "Did you write \"import ActionCreators from\" instead of \"import * as ActionCreators from\"?");
  }

  var keys = Object.keys(actionCreators);
  var boundActionCreators = {};
  for (var i = 0; i < keys.length; i++) {
    var key = keys[i];
    var actionCreator = actionCreators[key];
    if (typeof actionCreator === "function") {
      boundActionCreators[key] = bindArenaActionCreator(actionCreator, dispatch, sceneReducerKey);
    }
  }
  return boundActionCreators;
}