var _marked = /*#__PURE__*/regeneratorRuntime.mark(applySceneBundle);

import { ARENA_CURTAIN_SET_STATE } from "../actionTypes";
import { put, select, getContext } from "redux-saga/effects";
import { connect } from "react-redux";
import { createPropsPicker } from "../enhancedRedux";
import { sceneApplyRedux, sceneUpdateRedux } from "./sceneReduxSaga";
import { ARENA_STATETREE_NODE_DISABLE, ARENA_STATETREE_NODE_DELETE } from "../actionTypes";

/**
 * Scene of the synchronous load function, it does the following.
 * 1. load, add, replace reducer.
 * 2. Set the state of the corresponding scene.
 * 3. Run the corresponding scene of the saga, cancel the last scene of the task.
 * 4. Connect redux according to the incoming mapStateToProps and actions.
 * 
 * @param {any} { isInitial, arenaReducerDict, sceneBundle } 
 */
export function applySceneBundle(_ref) {
  var isInitial = _ref.isInitial,
      arenaReducerDict = _ref.arenaReducerDict,
      sceneBundle = _ref.sceneBundle,
      loadedCb = _ref.loadedCb;

  var arenaCurtainReducerKey, _ref2, curSceneBundle, reduxInfo, OldPlayingScene, mutableObj, newReduxInfo, newMutableObj, propsPicker, PlayingScene, displayName, newArenaState, arenaStore;

  return regeneratorRuntime.wrap(function applySceneBundle$(_context) {
    while (1) {
      switch (_context.prev = _context.next) {
        case 0:
          arenaCurtainReducerKey = arenaReducerDict._arenaCurtain.reducerKey;
          _context.next = 3;
          return select(function (state) {
            return state[arenaCurtainReducerKey];
          });

        case 3:
          _ref2 = _context.sent;
          curSceneBundle = _ref2.curSceneBundle;
          reduxInfo = _ref2.reduxInfo;
          OldPlayingScene = _ref2.PlayingScene;
          mutableObj = _ref2.mutableObj;

          mutableObj.isObsolete = true;
          newReduxInfo = void 0;

          if (!isInitial) {
            _context.next = 15;
            break;
          }

          return _context.delegateYield(sceneApplyRedux({
            arenaReducerDict: arenaReducerDict,
            state: sceneBundle.state,
            saga: sceneBundle.saga,
            actions: sceneBundle.actions,
            reducer: sceneBundle.reducer,
            options: sceneBundle.options || {}
          }), "t0", 12);

        case 12:
          newReduxInfo = _context.t0;
          _context.next = 17;
          break;

        case 15:
          return _context.delegateYield(sceneUpdateRedux({
            arenaReducerDict: arenaReducerDict,
            state: sceneBundle.state,
            saga: sceneBundle.saga,
            actions: sceneBundle.actions,
            reducer: sceneBundle.reducer,
            options: sceneBundle.options || {},
            curSceneBundle: curSceneBundle,
            reduxInfo: reduxInfo
          }), "t1", 16);

        case 16:
          newReduxInfo = _context.t1;

        case 17:
          newMutableObj = { isObsolete: false };
          _context.next = 20;
          return put({
            type: ARENA_CURTAIN_SET_STATE,
            _reducerKey: arenaCurtainReducerKey,
            state: {
              reduxInfo: newReduxInfo,
              mutableObj: newMutableObj
            }
          });

        case 20:
          propsPicker = createPropsPicker(sceneBundle.propsPicker, newReduxInfo, newMutableObj);
          PlayingScene = connect(propsPicker)(sceneBundle.Component);
          displayName = sceneBundle.Component.displayName || sceneBundle.Component.name || "Unknown";

          PlayingScene.displayName = "SceneConnect({reducerKey:" + newReduxInfo.reducerKey + ",Component:" + displayName + "})";
          newArenaState = {
            PlayingScene: PlayingScene,
            curSceneBundle: sceneBundle
          };
          _context.next = 27;
          return put({
            type: ARENA_CURTAIN_SET_STATE,
            _reducerKey: arenaCurtainReducerKey,
            state: newArenaState
          });

        case 27:
          loadedCb();

          if (!(newReduxInfo.reducerKey !== reduxInfo.reducerKey && reduxInfo.reducerKey != null)) {
            _context.next = 37;
            break;
          }

          _context.next = 31;
          return getContext("store");

        case 31:
          arenaStore = _context.sent;
          _context.next = 34;
          return put({
            type: ARENA_STATETREE_NODE_DISABLE,
            reducerKey: reduxInfo.reducerKey
          });

        case 34:
          arenaStore.removeReducer(reduxInfo.reducerKey);
          _context.next = 37;
          return put({
            type: ARENA_STATETREE_NODE_DELETE,
            reducerKey: reduxInfo.reducerKey
          });

        case 37:
        case "end":
          return _context.stop();
      }
    }
  }, _marked, this);
}