var _marked = /*#__PURE__*/regeneratorRuntime.mark(forkSagaWithContext),
    _marked2 = /*#__PURE__*/regeneratorRuntime.mark(sceneApplyRedux),
    _marked3 = /*#__PURE__*/regeneratorRuntime.mark(sceneUpdateRedux);

import { ARENA_SCENE_REPLACE_STATE } from "../actionTypes";
import { ARENA_SCENE_SET_STATE } from "../../actionTypes";
import { put, fork, select, getContext, setContext } from "redux-saga/effects";
import { bindActionCreators } from "redux";
import { bindArenaActionCreators } from "../enhancedRedux";
import { createSceneReducer, sceneReducerWrapper } from "../reducers";
import { addStateTreeNode, sceneAddReducer, sceneReplaceReducer, buildSceneReducerDict } from "../../utils";
import { getSwitchInitState } from "../reducers";

var defaultActions = {
  setState: function setState(state) {
    return { type: ARENA_SCENE_SET_STATE, state: state };
  }
};

function bindActions(actions, reducerKey, dispatch, isSceneActions) {
  if (isSceneActions === false) {
    return bindActionCreators(actions || defaultActions, dispatch);
  } else {
    return bindArenaActionCreators(actions || defaultActions, dispatch, reducerKey);
  }
}

function forkSagaWithContext(saga, ctx) {
  return regeneratorRuntime.wrap(function forkSagaWithContext$(_context) {
    while (1) {
      switch (_context.prev = _context.next) {
        case 0:
          _context.next = 2;
          return setContext(ctx);

        case 2:
          _context.next = 4;
          return fork(saga);

        case 4:
        case "end":
          return _context.stop();
      }
    }
  }, _marked, this);
}

function buildReducerFactory(reducer, state, isSceneReducer) {
  return isSceneReducer === false ? function (bindingReducerKey) {
    return createSceneReducer(reducer, state, bindingReducerKey);
  } : function (bindingReducerKey) {
    return createSceneReducer(reducer && sceneReducerWrapper(reducer), state, bindingReducerKey);
  };
}

function getParentReducerKey(arenaReducerDict) {
  return arenaReducerDict && arenaReducerDict._arenaCurtain && arenaReducerDict._arenaCurtain.reducerKey;
}

export function sceneApplyRedux(_ref) {
  var arenaReducerDict = _ref.arenaReducerDict,
      state = _ref.state,
      saga = _ref.saga,
      actions = _ref.actions,
      reducer = _ref.reducer,
      options = _ref.options;
  var arenaStore, reducerFactory, newReducerKey, bindedActions, newArenaReducerDict, newReduxInfo;
  return regeneratorRuntime.wrap(function sceneApplyRedux$(_context2) {
    while (1) {
      switch (_context2.prev = _context2.next) {
        case 0:
          _context2.next = 2;
          return getContext("store");

        case 2:
          arenaStore = _context2.sent;
          reducerFactory = buildReducerFactory(reducer, state, options.isSceneReducer);
          newReducerKey = sceneAddReducer(arenaStore, options.reducerKey, reducerFactory, state);
          bindedActions = bindActions(actions, newReducerKey, arenaStore.dispatch, options.isSceneActions);
          newArenaReducerDict = buildSceneReducerDict(arenaReducerDict, newReducerKey, options.vReducerKey, bindedActions);
          newReduxInfo = {
            reducerKey: newReducerKey,
            origArenaReducerDict: arenaReducerDict,
            actions: actions,
            options: options,
            saga: saga,
            bindedActions: bindedActions,
            arenaReducerDict: newArenaReducerDict
          };

          addStateTreeNode(arenaStore, getParentReducerKey(newReduxInfo.arenaReducerDict), newReducerKey);

          if (!saga) {
            _context2.next = 13;
            break;
          }

          _context2.next = 12;
          return fork(forkSagaWithContext, saga, {
            arenaReducerDict: newReduxInfo.arenaReducerDict
          });

        case 12:
          newReduxInfo.saga = _context2.sent;

        case 13:
          return _context2.abrupt("return", newReduxInfo);

        case 14:
        case "end":
          return _context2.stop();
      }
    }
  }, _marked2, this);
}

export function sceneUpdateRedux(_ref2) {
  var arenaReducerDict = _ref2.arenaReducerDict,
      state = _ref2.state,
      saga = _ref2.saga,
      actions = _ref2.actions,
      reducer = _ref2.reducer,
      options = _ref2.options,
      curSceneBundle = _ref2.curSceneBundle,
      reduxInfo = _ref2.reduxInfo;
  var newReducerKey, arenaStore, reducerFactory, newReduxInfo, oldState;
  return regeneratorRuntime.wrap(function sceneUpdateRedux$(_context3) {
    while (1) {
      switch (_context3.prev = _context3.next) {
        case 0:
          newReducerKey = reduxInfo.reducerKey;
          _context3.next = 3;
          return getContext("store");

        case 3:
          arenaStore = _context3.sent;
          reducerFactory = buildReducerFactory(reducer, state, options.isSceneReducer);
          newReduxInfo = Object.assign({}, reduxInfo, { options: options });

          if (!(options.reducerKey != null && options.reducerKey !== reduxInfo.reducerKey)) {
            _context3.next = 14;
            break;
          }

          _context3.next = 9;
          return select(function (state) {
            return state[reduxInfo.reducerKey];
          });

        case 9:
          oldState = _context3.sent;

          newReducerKey = sceneAddReducer(arenaStore, options.reducerKey, reducerFactory, state === curSceneBundle.state ? oldState : state);
          addStateTreeNode(arenaStore, newReducerKey);
          _context3.next = 21;
          break;

        case 14:
          if (!(options.reducerKey === reduxInfo.reducerKey)) {
            _context3.next = 18;
            break;
          }

          if (reducer !== curSceneBundle.reducer || options.isSceneReducer !== reduxInfo.options.isSceneReducer) {
            newReducerKey = sceneReplaceReducer(arenaStore, reduxInfo.reducerKey, reducerFactory, state === curSceneBundle.state ? null : state);
          } else if (state !== curSceneBundle.state) {
            arenaStore.dispatch({
              type: ARENA_SCENE_REPLACE_STATE,
              _sceneReducerKey: newReducerKey,
              state: state
            });
          }
          _context3.next = 21;
          break;

        case 18:
          if (!(state !== curSceneBundle.state)) {
            _context3.next = 21;
            break;
          }

          _context3.next = 21;
          return put({
            type: ARENA_SCENE_REPLACE_STATE,
            _sceneReducerKey: newReducerKey,
            state: state
          });

        case 21:
          newReduxInfo.reducerKey = newReducerKey;
          if (actions !== reduxInfo.actions || options.isSceneActions !== reduxInfo.options.isSceneActions) {
            newReduxInfo.actions = actions;
            newReduxInfo.bindedActions = bindActions(actions, newReducerKey, arenaStore.dispatch, options.isSceneActions);
          }
          if (newReducerKey !== reduxInfo.reducerKey || options.vReducerKey !== reduxInfo.options.vReducerKey || arenaReducerDict !== reduxInfo.origArenaReducerDict) {
            newReduxInfo.arenaReducerDict = buildSceneReducerDict(arenaReducerDict, newReducerKey, options.vReducerKey, newReduxInfo.actions);
          }

          if (!saga) {
            _context3.next = 28;
            break;
          }

          _context3.next = 27;
          return fork(forkSagaWithContext, saga, {
            arenaReducerDict: newReduxInfo.arenaReducerDict
          });

        case 27:
          newReduxInfo.saga = _context3.sent;

        case 28:
          return _context3.abrupt("return", newReduxInfo);

        case 29:
        case "end":
          return _context3.stop();
      }
    }
  }, _marked3, this);
}