function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

import React, { Component } from "react";
import PropTypes from "prop-types";
import { ARENA_CURTAIN_INIT_SAGA, ARENA_CURTAIN_CLEAR_REDUX } from "../../core/actionTypes";
import { createCurtainReducer } from "../../core/reducers";
import { addStateTreeNode, curtainAddReducer, buildCurtainReducerDict } from "../../utils";
import { arenaCurtainConnect } from "../SceneBundle";

function buildConnectedBundleComponent(reducerKey, store) {
  var sagaTaskPromise = new Promise(function (resolve) {
    return store.dispatch({
      type: ARENA_CURTAIN_INIT_SAGA,
      reducerKey: reducerKey,
      setSagaTask: resolve
    });
  });
  return arenaCurtainConnect(reducerKey, function () {
    return store.dispatch({
      type: ARENA_CURTAIN_CLEAR_REDUX,
      reducerKey: reducerKey,
      sagaTaskPromise: sagaTaskPromise
    });
  });
}

function getParentReducerKey(arenaReducerDict) {
  return arenaReducerDict && arenaReducerDict._arenaScene && arenaReducerDict._arenaScene.reducerKey;
}

var ArenaScene = function (_Component) {
  _inherits(ArenaScene, _Component);

  function ArenaScene() {
    _classCallCheck(this, ArenaScene);

    return _possibleConstructorReturn(this, _Component.apply(this, arguments));
  }

  ArenaScene.prototype.componentWillMount = function componentWillMount() {
    var _context = this.context,
        store = _context.store,
        arenaReducerDict = _context.arenaReducerDict;
    var _props = this.props,
        sceneBundle = _props.sceneBundle,
        sceneProps = _props.sceneProps,
        reducerKey = _props.reducerKey,
        vReducerKey = _props.vReducerKey;

    var newReducerKey = curtainAddReducer(store, reducerKey, createCurtainReducer);
    var parentReducerKey = getParentReducerKey(arenaReducerDict);
    addStateTreeNode(store, parentReducerKey, newReducerKey);
    var newArenaReducerDict = buildCurtainReducerDict(arenaReducerDict, newReducerKey, vReducerKey);
    var ConnectedBundleComponent = buildConnectedBundleComponent(newReducerKey, this.context.store);
    var connectedBundleElement = React.createElement(ConnectedBundleComponent, {
      sceneBundle: sceneBundle,
      sceneProps: sceneProps,
      arenaReducerDict: newArenaReducerDict
    });
    this.setState({
      parentReducerKey: parentReducerKey,
      arenaReducerDict: newArenaReducerDict,
      ConnectedBundleComponent: ConnectedBundleComponent,
      connectedBundleElement: connectedBundleElement
    });
  };

  ArenaScene.prototype.componentWillReceiveProps = function componentWillReceiveProps(nextProps, nextContext) {
    var refreshFlag = false;
    var state = Object.assign({}, this.state);
    var reducerKey = nextProps.reducerKey,
        vReducerKey = nextProps.vReducerKey,
        sceneBundle = nextProps.sceneBundle,
        sceneProps = nextProps.sceneProps;

    var curReducerKey = state.arenaReducerDict._arenaCurtain.reducerKey;
    var newReducerKey = curReducerKey;
    if (reducerKey != null && reducerKey !== curReducerKey) {
      refreshFlag = true;
      newReducerKey = curtainAddReducer(nextContext.store, reducerKey, createCurtainReducer);
      addStateTreeNode(store, this.state.parentReducerKey, newReducerKey);
      state.ConnectedBundleComponent = buildConnectedBundleComponent(newReducerKey, nextContext.store);
    }
    if (nextContext.arenaReducerDict !== this.context.arenaReducerDict || reducerKey !== this.props.reducerKey || vReducerKey !== this.props.vReducerKey || sceneBundle !== this.props.sceneBundle || sceneProps !== this.props.sceneBundle || refreshFlag === true) {
      refreshFlag = true;
      state.arenaReducerDict = buildCurtainReducerDict(nextContext.arenaReducerDict, newReducerKey, nextProps.vReducerKey);
      state.connectedBundleElement = React.createElement(state.ConnectedBundleComponent, {
        sceneBundle: sceneBundle,
        sceneProps: sceneProps,
        arenaReducerDict: state.arenaReducerDict
      });
    }
    this.setState(state);
  };

  ArenaScene.prototype.render = function render() {
    return this.state.connectedBundleElement;
  };

  return ArenaScene;
}(Component);

ArenaScene.contextTypes = {
  store: PropTypes.any,
  arenaReducerDict: PropTypes.object
};
ArenaScene.propTypes = {
  reducerKey: PropTypes.string,
  vReducerKey: PropTypes.string,
  sceneBundle: PropTypes.object,
  sceneProps: PropTypes.object
};
export default ArenaScene;