"use strict";

exports.__esModule = true;
exports.default = reducer;

var _actionTypes = require("../actionTypes.js");

var _getArenaInitState = require("./getArenaInitState");

var _getArenaInitState2 = _interopRequireDefault(_getArenaInitState);

var _immutable = require("immutable");

var _immutable2 = _interopRequireDefault(_immutable);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function addStateTreeNode(state, pReducerKey, reducerKey) {
  var stateTree = state.stateTree,
      stateTreeDict = state.stateTreeDict;

  var newStateTree = void 0,
      newStateTreeDict = void 0;
  var newNode = _immutable2.default.Map({
    pReducerKey: null,
    reducerKey: reducerKey,
    children: _immutable2.default.Map()
  });
  if (pReducerKey == null) {
    newStateTree = stateTree.set(reducerKey, newNode);
    newStateTreeDict = stateTreeDict.set(reducerKey, _immutable2.default.Map({ path: _immutable2.default.List([reducerKey]) }));
  } else {
    var pPath = stateTreeDict.getIn([pReducerKey, "path"]);
    var pNode = stateTree.getIn(pPath);
    newNode = newNode.set("pReducerKey", pNode.get("reducerKey"));
    var path = pPath.concat(["children", reducerKey]);
    var newPNode = pNode.setIn(["children", reducerKey], newNode);
    newStateTree = stateTree.setIn(pPath, newPNode);
    newStateTreeDict = stateTreeDict.set(reducerKey, _immutable2.default.Map({ path: path }));
  }
  return Object.assign({}, state, {
    stateTree: newStateTree,
    stateTreeDict: newStateTreeDict
  });
}

function getReducerKeysOfNode(node, breakChecker) {
  if (breakChecker && breakChecker(node)) return _immutable2.default.List();
  return node.get("children").map(function (child) {
    return getReducerKeysOfNode(child, breakChecker);
  }).reduce(function (prev, cur) {
    return prev.concat(cur);
  }, _immutable2.default.List([node.get("reducerKey")]));
}

function disableStateTreeNode(state, reducerKey) {
  var stateTreeDict = state.stateTreeDict,
      stateTree = state.stateTree;

  var path = stateTreeDict.getIn([reducerKey, "path"]);
  var node = stateTree.getIn(path);
  var obsoleteKeyList = getReducerKeysOfNode(node, function (node) {
    return stateTreeDict.getIn([node.get("reducerKey"), "isObsolete"]) === true;
  });
  var deltaDict = _immutable2.default.Map(obsoleteKeyList.map(function (tmpKey) {
    return [tmpKey, stateTreeDict.get(tmpKey).set("isObsolete", true)];
  }));
  var newStateTreeDict = stateTreeDict.merge(deltaDict);
  return Object.assign({}, state, {
    stateTreeDict: newStateTreeDict
  });
}

function findNodeUpward(path, stateTree, stateTreeDict, checker) {
  var node = stateTree.getIn(path);
  var dictItem = stateTreeDict.get(node.get("reducerKey"));
  if (checker(node, dictItem)) {
    if (path.count() > 1) {
      var nextPath = findNodeUpward(path.skipLast(2), stateTree, stateTreeDict, checker);
      return nextPath == null ? path : nextPath;
    } else {
      return path;
    }
  } else {
    return null;
  }
}

function deleteStateTreeNode(state, reducerKey) {
  var stateTree = state.stateTree,
      stateTreeDict = state.stateTreeDict;

  var path = stateTreeDict.getIn([reducerKey, "path"]);
  var node = stateTree.getIn(path);
  if (node.get("children").count() === 0) {
    var path4Del = findNodeUpward(path, stateTree, stateTreeDict, function (pNode, dictItem) {
      return dictItem.get("isObsolete") === true && pNode.get("children").count() === 1;
    });
    if (path4Del != null) {
      var obsoleteKeyList = getReducerKeysOfNode(stateTree.getIn(path4Del));
      var newStateTree = stateTree.deleteIn(path4Del);
      var newStateTreeDict = stateTreeDict.filterNot(function (_, key) {
        return obsoleteKeyList.includes(key);
      });
      return Object.assign({}, state, {
        stateTree: newStateTree,
        stateTreeDict: newStateTreeDict
      });
    }
  }
  return state;
}

function reducer() {
  var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : (0, _getArenaInitState2.default)();
  var action = arguments[1];

  switch (action.type) {
    case _actionTypes.ARENA_SET_STATE:
      return Object.assign({}, state, action.state);
    case _actionTypes.ARENA_REPLACE_STATE:
      return Object.assign({}, action.state);
    case _actionTypes.ARENA_GLOBAL_PROPSPICKER_LOCK:
      return Object.assign({ propsLock: action.lock });
    case _actionTypes.ARENA_STATETREE_NODE_ADD:
      return addStateTreeNode(state, action.pReducerKey, action.reducerKey);
    case _actionTypes.ARENA_STATETREE_NODE_DISABLE:
      return disableStateTreeNode(state, action.reducerKey);
    case _actionTypes.ARENA_STATETREE_NODE_DELETE:
      return deleteStateTreeNode(state, action.reducerKey);
    default:
      return state;
  }
}