"use strict";

exports.__esModule = true;

var _react = require("react");

var _react2 = _interopRequireDefault(_react);

var _propTypes = require("prop-types");

var _propTypes2 = _interopRequireDefault(_propTypes);

var _actionTypes = require("../../core/actionTypes");

var _reducers = require("../../core/reducers");

var _utils = require("../../utils");

var _SceneBundle = require("../SceneBundle");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

function buildConnectedBundleComponent(reducerKey, store) {
  var sagaTaskPromise = new Promise(function (resolve) {
    return store.dispatch({
      type: _actionTypes.ARENA_CURTAIN_INIT_SAGA,
      reducerKey: reducerKey,
      setSagaTask: resolve
    });
  });
  return (0, _SceneBundle.arenaCurtainConnect)(reducerKey, function () {
    return store.dispatch({
      type: _actionTypes.ARENA_CURTAIN_CLEAR_REDUX,
      reducerKey: reducerKey,
      sagaTaskPromise: sagaTaskPromise
    });
  });
}

function getParentReducerKey(arenaReducerDict) {
  return arenaReducerDict && arenaReducerDict._arenaScene && arenaReducerDict._arenaScene.reducerKey;
}

var ArenaScene = function (_Component) {
  _inherits(ArenaScene, _Component);

  function ArenaScene() {
    _classCallCheck(this, ArenaScene);

    return _possibleConstructorReturn(this, _Component.apply(this, arguments));
  }

  ArenaScene.prototype.componentWillMount = function componentWillMount() {
    var _context = this.context,
        store = _context.store,
        arenaReducerDict = _context.arenaReducerDict;
    var _props = this.props,
        sceneBundle = _props.sceneBundle,
        sceneProps = _props.sceneProps,
        reducerKey = _props.reducerKey,
        vReducerKey = _props.vReducerKey;

    var newReducerKey = (0, _utils.curtainAddReducer)(store, reducerKey, _reducers.createCurtainReducer);
    var parentReducerKey = getParentReducerKey(arenaReducerDict);
    (0, _utils.addStateTreeNode)(store, parentReducerKey, newReducerKey);
    var newArenaReducerDict = (0, _utils.buildCurtainReducerDict)(arenaReducerDict, newReducerKey, vReducerKey);
    var ConnectedBundleComponent = buildConnectedBundleComponent(newReducerKey, this.context.store);
    var connectedBundleElement = _react2.default.createElement(ConnectedBundleComponent, {
      sceneBundle: sceneBundle,
      sceneProps: sceneProps,
      arenaReducerDict: newArenaReducerDict
    });
    this.setState({
      parentReducerKey: parentReducerKey,
      arenaReducerDict: newArenaReducerDict,
      ConnectedBundleComponent: ConnectedBundleComponent,
      connectedBundleElement: connectedBundleElement
    });
  };

  ArenaScene.prototype.componentWillReceiveProps = function componentWillReceiveProps(nextProps, nextContext) {
    var refreshFlag = false;
    var state = Object.assign({}, this.state);
    var reducerKey = nextProps.reducerKey,
        vReducerKey = nextProps.vReducerKey,
        sceneBundle = nextProps.sceneBundle,
        sceneProps = nextProps.sceneProps;

    var curReducerKey = state.arenaReducerDict._arenaCurtain.reducerKey;
    var newReducerKey = curReducerKey;
    if (reducerKey != null && reducerKey !== curReducerKey) {
      refreshFlag = true;
      newReducerKey = (0, _utils.curtainAddReducer)(nextContext.store, reducerKey, _reducers.createCurtainReducer);
      (0, _utils.addStateTreeNode)(store, this.state.parentReducerKey, newReducerKey);
      state.ConnectedBundleComponent = buildConnectedBundleComponent(newReducerKey, nextContext.store);
    }
    if (nextContext.arenaReducerDict !== this.context.arenaReducerDict || reducerKey !== this.props.reducerKey || vReducerKey !== this.props.vReducerKey || sceneBundle !== this.props.sceneBundle || sceneProps !== this.props.sceneBundle || refreshFlag === true) {
      refreshFlag = true;
      state.arenaReducerDict = (0, _utils.buildCurtainReducerDict)(nextContext.arenaReducerDict, newReducerKey, nextProps.vReducerKey);
      state.connectedBundleElement = _react2.default.createElement(state.ConnectedBundleComponent, {
        sceneBundle: sceneBundle,
        sceneProps: sceneProps,
        arenaReducerDict: state.arenaReducerDict
      });
    }
    this.setState(state);
  };

  ArenaScene.prototype.render = function render() {
    return this.state.connectedBundleElement;
  };

  return ArenaScene;
}(_react.Component);

ArenaScene.contextTypes = {
  store: _propTypes2.default.any,
  arenaReducerDict: _propTypes2.default.object
};
ArenaScene.propTypes = {
  reducerKey: _propTypes2.default.string,
  vReducerKey: _propTypes2.default.string,
  sceneBundle: _propTypes2.default.object,
  sceneProps: _propTypes2.default.object
};
exports.default = ArenaScene;